@@ -70,7 +70,7 @@ static gint
 backend_get_fs_operations_name_comparator (gconstpointer a, gconstpointer b)
 {
   const struct fs_operations *ops = a;
-  return strcmp (ops->name, (gchar *) b);
+  return strcmp (GET_CLI_NAME (ops), (gchar *) b);
 }
 
 const struct fs_operations *
@@ -188,6 +188,7 @@ backend_tx_and_rx_sysex_transfer (struct backend *backend,
 				  gboolean free)
 {
   transfer->batch = FALSE;
+  transfer->err = 0;
 
   g_mutex_lock (&backend->mutex);
 
@@ -200,6 +201,7 @@ backend_tx_and_rx_sysex_transfer (struct backend *backend,
 	  transfer->raw = NULL;
 	}
     }
+
   if (!transfer->err)
     {
       backend_rx_sysex (backend, transfer);
