@@ -22,6 +22,42 @@
 #include "common.h"
 #include "sample.h"
 
+static const gchar *SYSEX_EXTS[] = { "syx", NULL };
+
+static void
+common_replace_chars (gchar *str, gchar x, gchar y)
+{
+  gchar *c = str;
+  while (*c)
+    {
+      if (*c == x)
+	{
+	  *c = y;
+	}
+      c++;
+    }
+}
+
+//These conversions depend on the OS but its safer and simpler to apply this restrictions to all of them.
+void
+common_to_os_sanitized_name (gchar *name)
+{
+  common_replace_chars (name, '/', '?');
+  common_replace_chars (name, '\\', '?');
+}
+
+const gchar **
+common_sysex_get_extensions ()
+{
+  return SYSEX_EXTS;
+}
+
+const gchar **
+common_get_all_extensions ()
+{
+  return NULL;
+}
+
 gchar *
 common_slot_get_upload_path (struct backend *backend,
 			     const struct fs_operations *ops,
@@ -80,7 +116,8 @@ common_print_item (struct item_iterator *iter, struct backend *backend,
   gchar *hsize = get_human_size (iter->item.size, FALSE);
   gint max_name_len = fs_ops->max_name_len ? fs_ops->max_name_len :
     DEFAULT_MAX_NAME_LEN;
-  gboolean info = (fs_ops->options & FS_OPTION_SHOW_INFO_COLUMN) != 0;
+  gboolean info = (fs_ops->options & FS_OPTION_SHOW_INFO_COLUMN) &&
+    *iter->item.object_info;
 
   if (fs_ops->options & FS_OPTION_SLOT_STORAGE)
     {
@@ -94,10 +131,11 @@ common_print_item (struct item_iterator *iter, struct backend *backend,
 	}
     }
 
-  printf ("%c %10s %.*s%s%-*s%s%s%s\n", iter->item.type, hsize, slot ? 10 : 0,
-	  slot, slot ? " " : "", max_name_len, iter->item.name,
-	  info ? " [ " : "", info ? iter->item.object_info : "",
-	  info ? " ]" : "");
+  printf ("%c %10s %.*s%s%-*s%s%s%s\n", iter->item.type, hsize,
+	  slot ? 10 : 0, slot, slot ? " " : "",
+	  info ? max_name_len : (gint) strlen (iter->item.name),
+	  iter->item.name, info ? " [ " : "",
+	  info ? iter->item.object_info : "", info ? " ]" : "");
 
   g_free (hsize);
   g_free (slot);
@@ -151,9 +189,7 @@ common_data_tx (struct backend *backend, GByteArray *msg,
 
   g_mutex_lock (&backend->mutex);
 
-  control->parts = 1;
-  control->part = 0;
-  job_control_set_progress (control, 0.0);
+  job_control_reset (control, 1);
 
   transfer.raw = msg;
   err = backend_tx_sysex (backend, &transfer);
@@ -182,8 +218,6 @@ common_data_tx_and_rx_part (struct backend *backend, GByteArray *tx_msg,
 {
   gint err = 0;
 
-  job_control_set_progress (control, 0.0);
-
   *rx_msg = backend_tx_and_rx_sysex (backend, tx_msg, -1);
   if (!*rx_msg)
     {
@@ -210,8 +244,7 @@ gint
 common_data_tx_and_rx (struct backend *backend, GByteArray *tx_msg,
 		       GByteArray **rx_msg, struct job_control *control)
 {
-  control->parts = 1;
-  control->part = 0;
+  job_control_reset (control, 1);
   return common_data_tx_and_rx_part (backend, tx_msg, rx_msg, control);
 }
 
@@ -223,6 +256,7 @@ common_slot_get_download_path_id_name (struct backend *backend,
 				       const gchar *name)
 {
   gchar *path;
+  const gchar *ext = GET_SAVE_EXT (ops, backend);
   GString *str = g_string_new (NULL);
 
   g_string_append_printf (str, "%s %s", backend->name, ops->name);
@@ -234,12 +268,17 @@ common_slot_get_download_path_id_name (struct backend *backend,
 
   if (name)
     {
+      gchar *sanitized_name = strdup (name);
+      common_to_os_sanitized_name (sanitized_name);
+
       g_string_append (str, " - ");
-      g_string_append (str, name);
+      g_string_append (str, sanitized_name);
+
+      g_free (sanitized_name);
     }
 
   g_string_append (str, ".");
-  g_string_append (str, ops->ext);
+  g_string_append (str, ext);
 
   path = path_chain (PATH_SYSTEM, dst_dir, str->str);
   g_string_free (str, TRUE);
@@ -345,8 +384,9 @@ common_system_get_download_path (struct backend *backend,
 				 const gchar *dst_dir, const gchar *src_path,
 				 struct idata *idata)
 {
+  const gchar *ext = GET_SAVE_EXT (ops, backend);
   GString *name_with_ext = g_string_new (NULL);
-  g_string_append_printf (name_with_ext, "%s.%s", idata->name, ops->ext);
+  g_string_append_printf (name_with_ext, "%s.%s", idata->name, ext);
   gchar *path = path_chain (PATH_SYSTEM, dst_dir, name_with_ext->str);
   g_string_free (name_with_ext, TRUE);
   return path;
