@@ -32,6 +32,10 @@ struct common_simple_read_dir_data
   guint32 last;
 };
 
+const gchar **common_sysex_get_extensions ();
+
+const gchar **common_get_all_extensions ();
+
 gchar *common_slot_get_upload_path (struct backend *backend,
 				    const struct fs_operations *ops,
 				    const gchar * dst_dir,
@@ -92,6 +96,8 @@ gchar *common_slot_get_download_path_nnn (struct backend *backend,
 					  const gchar * src_path,
 					  struct idata *idata);
 
+void common_to_os_sanitized_name (gchar * name);
+
 gchar *common_get_sanitized_name (const gchar * name, const gchar * alphabet,
 				  gchar defchar);
 
