@@ -21,8 +21,8 @@
 #ifndef CZ_H
 #define CZ_H
 
-#include "backend.h"
+#include "connector.h"
 
-gint cz_handshake (struct backend *);
+extern const struct connector CONNECTOR_CZ;
 
 #endif
