@@ -144,7 +144,7 @@ efactor_next_dentry (struct item_iterator *iter)
 
 static gint
 efactor_read_dir (struct backend *backend, struct item_iterator *iter,
-		  const gchar *dir, GSList *exts)
+		  const gchar *dir, const gchar **extensions)
 {
   GByteArray *tx_msg;
   GByteArray *rx_msg;
@@ -200,9 +200,7 @@ efactor_download (struct backend *backend, const gchar *path,
   struct efactor_data *data = backend->data;
   GByteArray *output;
 
-  control->parts = 1;
-  control->part = 0;
-  job_control_set_progress (control, 0.0);
+  job_control_reset (control, 1);
 
   if (!data->lines)
     {
@@ -385,12 +383,13 @@ efactor_get_slot (struct item *item, struct backend *backend)
   struct efactor_data *data = backend->data;
   if (data->type == EFACTOR_FACTOR)
     {
-      snprintf (slot, LABEL_MAX, "[%d:%d]", (item->id / 2) + 1,
-		(item->id % 2) + 1);
+      //This is a bit of a hack since not only are we showing the ID but also the bank-preset pair.
+      snprintf (slot, LABEL_MAX, "%02d [%d:%d]", item->id,
+		(item->id / 2) + 1, (item->id % 2) + 1);
     }
   else
     {
-      snprintf (slot, LABEL_MAX, "%d", item->id + 1);
+      snprintf (slot, LABEL_MAX, "%02d", item->id + 1);
     }
   return slot;
 }
@@ -402,7 +401,6 @@ static const struct fs_operations FS_EFACTOR_OPERATIONS = {
   .name = "preset",
   .gui_name = "Presets",
   .gui_icon = FS_ICON_SND,
-  .ext = BE_SYSEX_EXT,
   .max_name_len = EFACTOR_MAX_NAME_LEN,
   .readdir = efactor_read_dir,
   .print_item = common_print_item,
@@ -412,6 +410,7 @@ static const struct fs_operations FS_EFACTOR_OPERATIONS = {
   .get_slot = efactor_get_slot,
   .load = file_load,
   .save = file_save,
+  .get_exts = common_sysex_get_extensions,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = common_slot_get_download_path_nn,
   .select_item = common_midi_program_change
