@@ -30,11 +30,13 @@
 
 #define DEV_TAG_ID "id"
 #define DEV_TAG_NAME "name"
-#define DEV_TAG_ALIAS "alias"
 #define DEV_TAG_FILESYSTEMS "filesystems"
 #define DEV_TAG_STORAGE "storage"
 
-static const gchar *FS_TYPE_NAMES[] = { "+Drive", "RAM" };
+static const gchar *FS_TYPE_NAMES[] = { "+Drive", "RAM", NULL };
+
+static const gchar *FS_RAW_ANY_EXTS[] = { "raw", NULL };
+static const gchar *FS_DATA_ANY_EXTS[] = { "data", NULL };
 
 #define DATA_TRANSF_BLOCK_BYTES 0x2000
 #define OS_TRANSF_BLOCK_BYTES 0x800
@@ -141,6 +143,8 @@ static gint elektron_upload_raw_pst_pkg (struct backend *, const gchar *,
 
 static gboolean elektron_sample_file_exists (struct backend *, const gchar *);
 static gboolean elektron_raw_file_exists (struct backend *, const gchar *);
+const gchar **elektron_get_dev_exts (struct backend *,
+				     const struct fs_operations *);
 
 static const guint8 MSG_HEADER[] = { 0xf0, 0, 0x20, 0x3c, 0x10, 0 };
 
@@ -237,7 +241,8 @@ elektron_print_data (struct item_iterator *iter,
   gchar *hsize = get_human_size (iter->item.size, FALSE);
   gchar *slot = iter->item.id > 0 ?
     elektron_get_id_as_slot (&iter->item, backend) : strdup (" -1");
-  gboolean info = fs_ops->options & FS_OPTION_SHOW_INFO_COLUMN;
+  gboolean info = (fs_ops->options & FS_OPTION_SHOW_INFO_COLUMN) &&
+    *iter->item.object_info;
 
   printf ("%c %04x %d %d %10s %s %-*s%s%s%s\n", iter->item.type,
 	  data->operations, data->has_valid_data, data->has_metadata,
@@ -963,7 +968,7 @@ elektron_read_common_dir (struct backend *backend,
 static gint
 elektron_read_samples_dir (struct backend *backend,
 			   struct item_iterator *iter, const gchar *dir,
-			   GSList *extensions)
+			   const gchar **extensions)
 {
   return elektron_read_common_dir (backend, iter, dir,
 				   FS_SAMPLE_READ_DIR_REQUEST,
@@ -975,7 +980,7 @@ elektron_read_samples_dir (struct backend *backend,
 
 static gint
 elektron_read_raw_dir (struct backend *backend, struct item_iterator *iter,
-		       const gchar *dir, GSList *extensions)
+		       const gchar *dir, const gchar **extensions)
 {
   return elektron_read_common_dir (backend, iter, dir,
 				   FS_RAW_READ_DIR_REQUEST,
@@ -1606,7 +1611,7 @@ cleanup:
   return res;
 }
 
-static gint
+gint
 elektron_download_sample_part (struct backend *backend, const gchar *path,
 			       struct idata *sample,
 			       struct job_control *control)
@@ -1774,10 +1779,18 @@ elektron_get_storage_stats (struct backend *backend, guint8 type,
   gint8 op;
   guint64 *v;
   gint res = 0;
+  guint8 fsid;
   struct elektron_data *data = backend->data;
 
+  if (!(type & data->device_desc.storage))
+    {
+      return -EINVAL;
+    }
+
+  fsid = log2l (type);
+
   tx_msg = elektron_new_msg_uint8 (STORAGEINFO_REQUEST,
-				   sizeof (STORAGEINFO_REQUEST), type + 1);
+				   sizeof (STORAGEINFO_REQUEST), fsid + 1);
   rx_msg = elektron_tx_and_rx (backend, tx_msg);
   if (!rx_msg)
     {
@@ -1793,7 +1806,7 @@ elektron_get_storage_stats (struct backend *backend, guint8 type,
       return -EIO;
     }
 
-  snprintf (statfs->name, LABEL_MAX, "%s", FS_TYPE_NAMES[type]);
+  snprintf (statfs->name, LABEL_MAX, "%s", FS_TYPE_NAMES[fsid]);
   v = (guint64 *) & rx_msg->data[6];
   statfs->bfree = GUINT64_FROM_BE (*v);
   v = (guint64 *) & rx_msg->data[14];
@@ -1801,7 +1814,7 @@ elektron_get_storage_stats (struct backend *backend, guint8 type,
 
   free_msg (rx_msg);
 
-  return res ? res : (type < data->storage - 1);
+  return res ? res : type << 1 < data->device_desc.storage;
 }
 
 static gint
@@ -1888,12 +1901,17 @@ elektron_next_data_entry (struct item_iterator *iter)
 	{
 	  gchar metadata_path[PATH_MAX];
 	  struct idata output;
+	  struct job_control control;
+
+	  control.active = TRUE;
+	  control.callback = NULL;
+	  g_mutex_init (&control.mutex);
 
 	  snprintf (metadata_path, PATH_MAX, "%s/%d/%s", iter->dir,
 		    iter->item.id, FS_DATA_METADATA_FILE);
 	  debug_print (2, "Reading metadata from %s...", metadata_path);
 	  if (!elektron_download_data_snd (data->backend, metadata_path,
-					   &output, NULL))
+					   &output, &control))
 	    {
 	      gchar *s;
 	      gboolean first = TRUE;
@@ -1931,7 +1949,7 @@ elektron_next_data_entry (struct item_iterator *iter)
 not_found:
   iter->item.type = ITEM_TYPE_FILE;
   iter->item.name[0] = 0;
-  iter->item.size = -1;
+  iter->item.size = 0;
   iter->item.id++;
   data->operations = 0;
   data->has_valid_data = 0;
@@ -1998,7 +2016,7 @@ elektron_read_data_dir_prefix (struct backend *backend,
 static gint
 elektron_read_data_dir_any (struct backend *backend,
 			    struct item_iterator *iter, const gchar *dir,
-			    GSList *extensions)
+			    const gchar **extensions)
 {
   return elektron_read_data_dir_prefix (backend, iter, dir, NULL,
 					ITER_MODE_DATA, -1);
@@ -2007,7 +2025,7 @@ elektron_read_data_dir_any (struct backend *backend,
 static gint
 elektron_read_data_dir_prj (struct backend *backend,
 			    struct item_iterator *iter, const gchar *dir,
-			    GSList *extensions)
+			    const gchar **extensions)
 {
   return elektron_read_data_dir_prefix (backend, iter, dir,
 					FS_DATA_PRJ_PREFIX, ITER_MODE_DATA,
@@ -2017,7 +2035,7 @@ elektron_read_data_dir_prj (struct backend *backend,
 static gint
 elektron_read_data_dir_snd (struct backend *backend,
 			    struct item_iterator *iter, const gchar *dir,
-			    GSList *extensions)
+			    const gchar **extensions)
 {
   return elektron_read_data_dir_prefix (backend, iter, dir,
 					FS_DATA_SND_PREFIX,
@@ -2027,10 +2045,10 @@ elektron_read_data_dir_snd (struct backend *backend,
 static gint
 elektron_read_data_dir_pst (struct backend *backend,
 			    struct item_iterator *iter, const gchar *dir,
-			    GSList *extensions)
+			    const gchar **extensions)
 {
   struct elektron_data *data = backend->data;
-  gint32 slots = data->device_desc.id == 32 ? 512 : 128;	//Analog Heat +FX has 512 presets
+  gint32 slots = data->device_desc.id == ELEKTRON_AH_FX_ID ? 512 : 128;	//Analog Heat +FX has 512 presets
   return elektron_read_data_dir_prefix (backend, iter, dir,
 					FS_DATA_PST_PREFIX,
 					ITER_MODE_DATA, slots);
@@ -2253,10 +2271,6 @@ elektron_open_datum (struct backend *backend, const gchar *path,
     }
 
   tx_msg = elektron_new_msg (data, len);
-  if (!tx_msg)
-    {
-      return -ENOMEM;
-    }
 
   path_cp1252 = elektron_get_cp1252 (path);
 
@@ -2351,10 +2365,6 @@ elektron_close_datum (struct backend *backend,
     }
 
   tx_msg = elektron_new_msg (data, len);
-  if (!tx_msg)
-    {
-      return -ENOMEM;
-    }
 
   jidbe = g_htonl (jid);
   g_byte_array_append (tx_msg, (guchar *) & jidbe, sizeof (guint32));
@@ -2447,16 +2457,9 @@ elektron_download_data_prefix (struct backend *backend, const gchar *path,
 
   content = g_byte_array_sized_new (4 * MI);
 
-  if (control)
-    {
-      g_mutex_lock (&control->mutex);
-      active = control->active;
-      g_mutex_unlock (&control->mutex);
-    }
-  else
-    {
-      active = TRUE;
-    }
+  g_mutex_lock (&control->mutex);
+  active = control->active;
+  g_mutex_unlock (&control->mutex);
 
   jidbe = g_htonl (jid);
   err = 0;
@@ -2523,23 +2526,17 @@ elektron_download_data_prefix (struct backend *backend, const gchar *path,
       free_msg (rx_msg);
       seq++;
 
-      if (control)
-	{
-	  job_control_set_progress (control, status / 1000.0);
-	  g_mutex_lock (&control->mutex);
-	  active = control->active;
-	  g_mutex_unlock (&control->mutex);
-	}
+      job_control_set_progress (control, status / 1000.0);
+      g_mutex_lock (&control->mutex);
+      active = control->active;
+      g_mutex_unlock (&control->mutex);
 
       usleep (BE_REST_TIME_US);
     }
 
   if (active)
     {
-      if (control)
-	{
-	  job_control_set_progress (control, 1.0);
-	}
+      job_control_set_progress (control, 1.0);
       idata_init (data, content, NULL, NULL);
     }
   else
@@ -2654,7 +2651,7 @@ elektron_download_pkg (struct backend *backend, const gchar *path,
     }
 
   ret = package_receive_pkg_resources (&pkg, path, control, backend, download,
-				       elektron_download_sample_part, type);
+				       type);
   ret = ret || package_end (&pkg, output);
 
   package_destroy (&pkg);
@@ -2686,16 +2683,6 @@ elektron_get_upload_path_smplrw (struct backend *backend,
   return path;
 }
 
-static gchar *
-elektron_get_dev_ext (struct backend *backend,
-		      const struct fs_operations *ops)
-{
-  struct elektron_data *data = backend->data;
-  gchar *ext = g_malloc (LABEL_MAX);
-  snprintf (ext, LABEL_MAX, "%s%s", data->device_desc.alias, ops->ext);
-  return ext;
-}
-
 // As Elektron devices provide their own file extension and the file content is
 // not just SysEx, it is not needed to indicate in the filename the type of
 // device, filesystem or id.
@@ -2706,7 +2693,7 @@ elektron_get_download_path (struct backend *backend,
 			    const gchar *dst_dir, const gchar *src_path,
 			    struct idata *any)
 {
-  gchar *path, *name, *dl_ext, *src_fpath;
+  gchar *path, *name, *src_fpath;
   const gchar *md_ext, *ext = filename_get_ext (src_path);
 
   // Examples:
@@ -2730,11 +2717,10 @@ elektron_get_download_path (struct backend *backend,
   if (name)
     {
       GString *filename = g_string_new (NULL);
-      dl_ext = elektron_get_dev_ext (backend, ops);
+      const gchar *dl_ext = ops->get_exts (backend, ops)[0];
       g_string_append_printf (filename, "%s.%s%s", name, dl_ext, md_ext);
       path = path_chain (PATH_SYSTEM, dst_dir, filename->str);
       g_free (name);
-      g_free (dl_ext);
       g_string_free (filename, TRUE);
     }
   else
@@ -2776,7 +2762,6 @@ elektron_upload_data_prefix (struct backend *backend, const gchar *path,
     }
 
   path_w_prefix = elektron_add_prefix_to_path (path, prefix);
-
   err = elektron_open_datum (backend, path_w_prefix, &jid, O_WRONLY,
 			     array->len);
   g_free (path_w_prefix);
@@ -2791,16 +2776,10 @@ elektron_upload_data_prefix (struct backend *backend, const gchar *path,
 
   seq = 0;
   offset = 0;
-  if (control)
-    {
-      g_mutex_lock (&control->mutex);
-      active = control->active;
-      g_mutex_unlock (&control->mutex);
-    }
-  else
-    {
-      active = TRUE;
-    }
+
+  g_mutex_lock (&control->mutex);
+  active = control->active;
+  g_mutex_unlock (&control->mutex);
 
   while (offset < array->len && active)
     {
@@ -2937,46 +2916,37 @@ elektron_upload_pkg (struct backend *backend, const gchar *path,
   return ret;
 }
 
-GSList *
+const gchar **
 elektron_get_dev_exts (struct backend *backend,
 		       const struct fs_operations *ops)
-{
-  gchar *ext = elektron_get_dev_ext (backend, ops);
-  return g_slist_append (NULL, ext);
-}
-
-GSList *
-elektron_get_dev_exts_pst (struct backend *backend,
-			   const struct fs_operations *ops)
 {
   struct elektron_data *data = backend->data;
-  GSList *exts = elektron_get_dev_exts (backend, ops);
-  if (data->device_desc.id == 32)	//AH +FX
+  struct fs_desc *fs_desc = data->device_desc.fs_descs;
+
+  for (guint i = 0; i < data->device_desc.fs_descs_len; i++)
     {
-      exts = g_slist_append (exts, strdup ("ahpst"));
+      if (!strcmp (fs_desc->name, ops->name))
+	{
+	  return (const gchar **) fs_desc->extensions;
+	}
+      fs_desc++;
     }
-  return exts;
+
+  return NULL;
 }
 
-GSList *
-elektron_get_dev_exts_prj (struct backend *backend,
-			   const struct fs_operations *ops)
+const gchar **
+elektron_get_data_any_exts (struct backend *backend,
+			    const struct fs_operations *ops)
 {
-  struct elektron_data *data = backend->data;
-  GSList *exts = elektron_get_dev_exts (backend, ops);
-  if (data->device_desc.id == 42)	//Digitakt II
-    {
-      exts = g_slist_append (exts, strdup ("dtprj"));
-    }
-  return exts;
+  return FS_DATA_ANY_EXTS;
 }
 
-GSList *
-elektron_get_dt2_pst_exts (struct backend *backend,
+const gchar **
+elektron_get_raw_any_exts (struct backend *backend,
 			   const struct fs_operations *ops)
 {
-  GSList *exts = elektron_get_dev_exts (backend, ops);
-  return g_slist_append (exts, strdup ("dtsnd"));
+  return FS_RAW_ANY_EXTS;
 }
 
 gint
@@ -3043,7 +3013,7 @@ elektron_get_sample_path_from_hash_size (struct backend *backend,
   return path;
 }
 
-gint
+static gint
 elektron_sample_save (const gchar *path, struct idata *sample,
 		      struct job_control *control)
 {
@@ -3058,7 +3028,6 @@ static const struct fs_operations FS_SAMPLES_OPERATIONS = {
   .name = "sample",
   .gui_name = "Samples",
   .gui_icon = FS_ICON_WAVE,
-  .ext = "wav",
   .max_name_len = ELEKTRON_NAME_MAX_LEN,
   .readdir = elektron_read_samples_dir,
   .file_exists = elektron_sample_file_exists,
@@ -3080,7 +3049,6 @@ static const struct fs_operations FS_RAW_ANY_OPERATIONS = {
   .id = FS_RAW_ALL,
   .options = 0,
   .name = "raw",
-  .ext = "raw",
   .max_name_len = ELEKTRON_NAME_MAX_LEN,
   .readdir = elektron_read_raw_dir,
   .file_exists = elektron_raw_file_exists,
@@ -3093,7 +3061,7 @@ static const struct fs_operations FS_RAW_ANY_OPERATIONS = {
   .upload = elektron_upload_raw,
   .load = file_load,
   .save = file_save,
-  .get_exts = elektron_get_dev_exts,
+  .get_exts = elektron_get_raw_any_exts,
   .get_upload_path = elektron_get_upload_path_smplrw,
   .get_download_path = elektron_get_download_path
 };
@@ -3101,10 +3069,10 @@ static const struct fs_operations FS_RAW_ANY_OPERATIONS = {
 static const struct fs_operations FS_RAW_PRESETS_OPERATIONS = {
   .id = FS_RAW_PRESETS,
   .options = FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_ALLOW_SEARCH,
-  .name = "preset",
+  .name = "preset-raw",
+  .cli_name = "preset",
   .gui_name = "Presets",
   .gui_icon = FS_ICON_SND,
-  .ext = "pst",
   .max_name_len = ELEKTRON_NAME_MAX_LEN,
   .readdir = elektron_read_raw_dir,
   .file_exists = elektron_raw_file_exists,
@@ -3126,7 +3094,6 @@ static const struct fs_operations FS_DATA_ANY_OPERATIONS = {
   .id = FS_DATA_ANY,
   .options = FS_OPTION_SLOT_STORAGE,
   .name = "data",
-  .ext = "data",
   .readdir = elektron_read_data_dir_any,
   .print_item = elektron_print_data,
   .delete = elektron_clear_data_item_any,
@@ -3139,7 +3106,7 @@ static const struct fs_operations FS_DATA_ANY_OPERATIONS = {
   .get_slot = elektron_get_id_as_slot,
   .load = file_load,
   .save = file_save,
-  .get_exts = elektron_get_dev_exts,
+  .get_exts = elektron_get_data_any_exts,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = elektron_get_download_path
 };
@@ -3151,7 +3118,6 @@ static const struct fs_operations FS_DATA_PRJ_OPERATIONS = {
   .name = "project",
   .gui_name = "Projects",
   .gui_icon = FS_ICON_PRJ,
-  .ext = "prj",
   .readdir = elektron_read_data_dir_prj,
   .print_item = elektron_print_data,
   .delete = elektron_clear_data_item_prj,
@@ -3164,7 +3130,7 @@ static const struct fs_operations FS_DATA_PRJ_OPERATIONS = {
   .get_slot = elektron_get_id_as_slot,
   .load = file_load,
   .save = file_save,
-  .get_exts = elektron_get_dev_exts_prj,
+  .get_exts = elektron_get_dev_exts,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = elektron_get_download_path
 };
@@ -3177,7 +3143,6 @@ static const struct fs_operations FS_DATA_SND_OPERATIONS = {
   .name = "sound",
   .gui_name = "Sounds",
   .gui_icon = FS_ICON_SND,
-  .ext = "snd",
   .readdir = elektron_read_data_dir_snd,
   .print_item = elektron_print_data,
   .delete = elektron_clear_data_item_snd,
@@ -3202,7 +3167,6 @@ static const struct fs_operations FS_DATA_PST_OPERATIONS = {
   .name = "preset",
   .gui_name = "Presets",
   .gui_icon = FS_ICON_SND,
-  .ext = "pst",
   .readdir = elektron_read_data_dir_pst,
   .print_item = elektron_print_data,
   .delete = elektron_clear_data_item_pst,
@@ -3215,7 +3179,7 @@ static const struct fs_operations FS_DATA_PST_OPERATIONS = {
   .get_slot = elektron_get_id_as_slot,
   .load = file_load,
   .save = file_save,
-  .get_exts = elektron_get_dev_exts_pst,
+  .get_exts = elektron_get_dev_exts,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = elektron_get_download_path
 };
@@ -3224,10 +3188,10 @@ static const struct fs_operations FS_SAMPLES_STEREO_OPERATIONS = {
   .id = FS_SAMPLES_STEREO,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_ALLOW_SEARCH,
-  .name = "sample",
+  .name = "sample-stereo",
+  .cli_name = "sample",
   .gui_name = "Samples",
   .gui_icon = FS_ICON_WAVE,
-  .ext = "wav",
   .max_name_len = ELEKTRON_NAME_MAX_LEN,
   .readdir = elektron_read_samples_dir,
   .file_exists = elektron_sample_file_exists,
@@ -3245,15 +3209,15 @@ static const struct fs_operations FS_SAMPLES_STEREO_OPERATIONS = {
   .get_download_path = common_system_get_download_path
 };
 
-static const struct fs_operations FS_DATA_DT2_PST_OPERATIONS = {
-  .id = FS_DATA_DT2_PST,
+static const struct fs_operations FS_DATA_TAKT_II_PST_OPERATIONS = {
+  .id = FS_DATA_TAKT_II_PST,
   .options = FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_SIZE_COLUMN |
     FS_OPTION_SHOW_SLOT_COLUMN | FS_OPTION_SHOW_INFO_COLUMN |
     FS_OPTION_ALLOW_SEARCH,
-  .name = "preset",
+  .name = "preset-takt-ii",
+  .cli_name = "preset",
   .gui_name = "Presets",
   .gui_icon = FS_ICON_SND,
-  .ext = "pst",
   .readdir = elektron_read_data_dir_snd,
   .print_item = elektron_print_data,
   .delete = elektron_clear_data_item_snd,
@@ -3266,7 +3230,7 @@ static const struct fs_operations FS_DATA_DT2_PST_OPERATIONS = {
   .get_slot = elektron_get_id_as_slot,
   .load = file_load,
   .save = file_save,
-  .get_exts = elektron_get_dt2_pst_exts,	//Backwards compatible Digitakt I
+  .get_exts = elektron_get_dev_exts,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = elektron_get_download_path
 };
@@ -3275,42 +3239,32 @@ static const struct fs_operations *FS_OPERATIONS[] = {
   &FS_SAMPLES_OPERATIONS, &FS_RAW_ANY_OPERATIONS, &FS_RAW_PRESETS_OPERATIONS,
   &FS_DATA_ANY_OPERATIONS, &FS_DATA_PRJ_OPERATIONS, &FS_DATA_SND_OPERATIONS,
   &FS_DATA_PST_OPERATIONS, &FS_SAMPLES_STEREO_OPERATIONS,
-  &FS_DATA_DT2_PST_OPERATIONS, NULL
+  &FS_DATA_TAKT_II_PST_OPERATIONS, NULL
 };
 
-static gint
-elektron_configure_device (struct backend *backend, guint8 id)
+gint
+elektron_configure_device_from_file (struct backend *backend, guint8 id,
+				     const gchar *filename)
 {
-  gint err, devices;
-  guint32 filesystems = 0;
+  gint err, devices, storage;
   JsonParser *parser;
   JsonReader *reader;
-  gchar *devices_filename;
+  gchar **members;
+  gchar **name;
   GError *error = NULL;
   struct elektron_data *data = backend->data;
 
   parser = json_parser_new ();
 
-  devices_filename = get_user_dir (CONF_DIR DEVICES_FILE);
-
-  if (!json_parser_load_from_file (parser, devices_filename, &error))
+  if (!json_parser_load_from_file (parser, filename, &error))
     {
       debug_print (1, "%s", error->message);
       g_clear_error (&error);
-
-      g_free (devices_filename);
-      devices_filename = strdup (DATADIR DEVICES_FILE);
-
-      if (!json_parser_load_from_file (parser, devices_filename, &error))
-	{
-	  error_print ("%s", error->message);
-	  g_clear_error (&error);
-	  err = -ENODEV;
-	  goto cleanup_parser;
-	}
+      err = -ENODEV;
+      goto cleanup_parser;
     }
 
-  debug_print (1, "Using %s...", devices_filename);
+  debug_print (1, "Parsing %s...", filename);
 
   reader = json_reader_new (json_parser_get_root (parser));
   if (!reader)
@@ -3336,7 +3290,7 @@ elektron_configure_device (struct backend *backend, guint8 id)
     }
 
   err = -ENODEV;
-  for (int i = 0; i < devices; i++)
+  for (guint i = 0; i < devices; i++)
     {
       if (!json_reader_read_element (reader, i))
 	{
@@ -3361,17 +3315,6 @@ elektron_configure_device (struct backend *backend, guint8 id)
       err = 0;
       debug_print (1, "Device %d found", id);
 
-      if (!json_reader_read_member (reader, DEV_TAG_ALIAS))
-	{
-	  error_print ("Cannot read member '%s'. Stopping...", DEV_TAG_ALIAS);
-	  json_reader_end_element (reader);
-	  err = -ENODEV;
-	  break;
-	}
-      snprintf (data->device_desc.alias, LABEL_MAX, "%s",
-		json_reader_get_string_value (reader));
-      json_reader_end_member (reader);
-
       if (!json_reader_read_member (reader, DEV_TAG_NAME))
 	{
 	  error_print ("Cannot read member '%s'. Stopping...", DEV_TAG_NAME);
@@ -3391,7 +3334,72 @@ elektron_configure_device (struct backend *backend, guint8 id)
 	  err = -ENODEV;
 	  break;
 	}
-      filesystems = json_reader_get_int_value (reader);
+      members = json_reader_list_members (reader);
+      name = members;
+      data->device_desc.fs_descs_len = 0;
+      backend->fs_ops = NULL;
+      while (*name)
+	{
+	  const struct fs_operations **fs_ops = FS_OPERATIONS;
+	  while (*fs_ops)
+	    {
+	      const gchar *fs_name = (*fs_ops)->name;
+	      if (!strcmp (fs_name, *name))
+		{
+		  debug_print (2, "Reading '%s' filesystem...", fs_name);
+		  backend->fs_ops = g_slist_append (backend->fs_ops,
+						    (gpointer) * fs_ops);
+		  break;
+		}
+	      fs_ops++;
+	    }
+
+	  if (*fs_ops)
+	    {
+	      if (data->device_desc.fs_descs_len == ELEKTRON_MAX_FS)
+		{
+		  error_print ("Too many filesystems");
+		  break;
+		}
+	      struct fs_desc *fs_desc =
+		&data->device_desc.fs_descs[data->device_desc.fs_descs_len];
+	      snprintf (fs_desc->name, LABEL_MAX, "%s", *name);
+	      data->device_desc.fs_descs_len++;
+
+	      json_reader_read_member (reader, *name);
+
+	      gint j = 0;
+
+	      if (!json_reader_get_null_value (reader))
+		{
+		  gint extensions = json_reader_count_elements (reader);
+		  for (; j < extensions && j < ELEKTRON_MAX_EXTENSIONS - 1;
+		       j++)
+		    {
+		      json_reader_read_element (reader, j);
+
+		      const gchar *ext_name =
+			json_reader_get_string_value (reader);
+		      debug_print (2, "Adding '%s' extension...", ext_name);
+		      fs_desc->extensions[j] = strdup (ext_name);
+
+		      json_reader_end_element (reader);
+		    }
+		}
+
+	      fs_desc->extensions[j] = NULL;
+
+	      json_reader_end_member (reader);
+	    }
+	  else
+	    {
+	      error_print ("Filesystem '%s' not found", *name);
+	    }
+
+	  name++;
+	}
+
+      g_strfreev (members);
       json_reader_end_member (reader);
 
       if (!json_reader_read_member (reader, DEV_TAG_STORAGE))
@@ -3402,29 +3410,40 @@ elektron_configure_device (struct backend *backend, guint8 id)
 	  err = -ENODEV;
 	  break;
 	}
-      data->storage = json_reader_get_int_value (reader);
-      json_reader_end_member (reader);
+      data->device_desc.storage = 0;
+      storage = json_reader_count_elements (reader);
+      if (storage > ELEKTRON_MAX_STORAGE)
+	{
+	  error_print ("Too many storage (%d)", storage);
+	  storage = ELEKTRON_MAX_STORAGE;
+	}
+      for (guint j = 0; j < storage; j++)
+	{
+	  json_reader_read_element (reader, j);
+	  const gchar *storage_name = json_reader_get_string_value (reader);
+	  const gchar **stname = FS_TYPE_NAMES;
+	  guint id = 1;
+	  while (*stname)
+	    {
+	      if (strcmp (*stname, storage_name) == 0)
+		{
+		  data->device_desc.storage |= id;
+		  break;
+		}
+	      stname++;
+	      id <<= 1;
+	    }
+	  json_reader_end_element (reader);
+	}
+      json_reader_end_element (reader);
 
       break;
     }
 
-  backend->fs_ops = NULL;
-  const struct fs_operations **fs_ops = FS_OPERATIONS;
-  while (*fs_ops)
-    {
-      if ((*fs_ops)->id & filesystems)
-	{
-	  backend->fs_ops = g_slist_append (backend->fs_ops,
-					    (gpointer) * fs_ops);
-	}
-      fs_ops++;
-    }
-
 cleanup_reader:
   g_object_unref (reader);
 cleanup_parser:
   g_object_unref (parser);
-  g_free (devices_filename);
   if (err)
     {
       data->device_desc.id = -1;
@@ -3432,6 +3451,24 @@ cleanup_parser:
   return err;
 }
 
+static gint
+elektron_configure_device (struct backend *backend, guint8 id)
+{
+
+  gchar *filename = get_user_dir (CONF_DIR DEVICES_FILE);
+  gint err = elektron_configure_device_from_file (backend, id, filename);
+  g_free (filename);
+
+  if (err)
+    {
+      filename = strdup (DATADIR DEVICES_FILE);
+      err = elektron_configure_device_from_file (backend, id, filename);
+      g_free (filename);
+    }
+
+  return err;
+}
+
 GByteArray *
 elektron_ping (struct backend *backend)
 {
@@ -3454,6 +3491,36 @@ elektron_ping (struct backend *backend)
   return rx_msg;
 }
 
+void
+elektron_destroy_data (struct backend *backend)
+{
+  struct elektron_data *data = backend->data;
+  struct device_desc *device_desc = &data->device_desc;
+  struct fs_desc *fs_desc = device_desc->fs_descs;
+
+  debug_print (1, "Destroying backend elektron data...");
+
+
+  for (guint i = 0; i < device_desc->fs_descs_len; i++)
+    {
+      gchar **ext = fs_desc->extensions;
+      for (guint j = 0; j < ELEKTRON_MAX_EXTENSIONS; j++)
+	{
+	  if (!*ext)
+	    {
+	      break;
+	    }
+
+	  g_free (*ext);
+	  ext++;
+	}
+      fs_desc++;
+    }
+
+  g_free (backend->data);
+  backend->data = NULL;
+}
+
 static gint
 elektron_handshake (struct backend *backend)
 {
@@ -3517,10 +3584,10 @@ elektron_handshake (struct backend *backend)
 
   g_free (overbridge_name);
 
-  backend->destroy_data = backend_destroy_data;
+  backend->destroy_data = elektron_destroy_data;
   backend->upgrade_os = elektron_upgrade_os;
-  backend->get_storage_stats = data->storage ? elektron_get_storage_stats :
-    NULL;
+  backend->get_storage_stats =
+    data->device_desc.storage ? elektron_get_storage_stats : NULL;
 
   return 0;
 }
