@@ -48,7 +48,7 @@
 
 #define METADATA_TAG_SAMPLE_REFS "sound_tags"
 
-#define MAX_PACKAGE_LEN (64 * 1024 * 1024)
+#define MAX_PACKAGE_LEN (512 * 1024 * 1024)
 #define MAX_MANIFEST_LEN (128 * 1024)
 #define MANIFEST_FILENAME "manifest.json"
 
@@ -104,7 +104,7 @@ package_get_tags_from_snd_metadata (GByteArray *metadata)
 {
   JsonParser *parser;
   JsonReader *reader;
-  GError *error;
+  GError *error = NULL;
   GSList *tags = NULL;
 
   parser = json_parser_new ();
@@ -291,7 +291,7 @@ package_add_manifest (struct package *pkg)
 	}
     }
 
-  if (pkg->manifest->tags)	// PKG_FILE_TYPE_DATA_SOUND or PKG_FILE_TYPE_DATA_PRESET
+  if (pkg->manifest->tags)	// PKG_FILE_TYPE_DATA_SOUND
     {
       json_builder_set_member_name (builder, PKG_TAG_METAINFO);
       json_builder_begin_object (builder);
@@ -484,16 +484,15 @@ package_close (struct package *pkg)
   package_destroy (pkg);
 }
 
+//This function is just used as the upper bound estimation.
 static gint
-packaget_get_pkg_sample_slots (struct backend *backend)
+package_get_max_sample_slots (struct backend *backend)
 {
   struct elektron_data *data = backend->data;
-  gint slots = 127;		// slot 0 is never used
-  if (data->device_desc.id == 42)	// Digitakt II has 8 banks (1026)
-    {
-      slots *= 8;
-    }
-  return slots;
+  // Slot 0 is never used.
+  // Digitakt and similar devices have 128 slots.
+  // Digitakt II has 8 banks (1024).
+  return data->device_desc.id == ELEKTRON_DIGITAKT_II_ID ? 1023 : 127;
 }
 
 gint
@@ -502,25 +501,39 @@ package_receive_pkg_resources (struct package *pkg,
 			       struct job_control *control,
 			       struct backend *backend,
 			       fs_remote_file_op download_data,
-			       fs_remote_file_op download_sample,
 			       enum package_type type)
 {
   gint ret, i, elements;
   JsonParser *parser;
   JsonReader *reader;
   gint64 hash, size;
-  GError *error;
+  GError *error = NULL;
   gchar *sample_path, *metadata_path;
   struct package_resource *pkg_resource;
   GString *package_resource_path;
   struct idata metadata_file, payload_file, sample_file, file;
+  struct elektron_data *data = backend->data;
+
+  pkg->manifest->tags = NULL;
+
+  job_control_reset (control, 1);	//payload
+
+  if ((type == PKG_FILE_TYPE_DATA_PROJECT &&
+       data->device_desc.id != ELEKTRON_ANALOG_RYTM_ID &&
+       data->device_desc.id != ELEKTRON_DIGITAKT_ID &&
+       data->device_desc.id != ELEKTRON_ANALOG_RYTM_MKII_ID &&
+       data->device_desc.id != ELEKTRON_MODEL_SAMPLES_ID &&
+       data->device_desc.id != ELEKTRON_DIGITAKT_II_ID) ||
+      type == PKG_FILE_TYPE_DATA_PRESET)
+    {
+      goto get_payload;
+    }
+
+  control->parts += 1 + package_get_max_sample_slots (backend);	// ... plus metadata and sample slots.
 
   metadata_path = path_chain (PATH_INTERNAL, payload_path,
 			      FS_DATA_METADATA_FILE);
   debug_print (1, "Getting metadata from %s...", metadata_path);
-  control->parts = 2 + packaget_get_pkg_sample_slots (backend);	// main, metadata and sample slots.
-  control->part = 0;
-  job_control_set_progress (control, 0.0);
 
   ret = download_data (backend, metadata_path, &metadata_file, control);
   if (ret)
@@ -552,7 +565,7 @@ package_receive_pkg_resources (struct package *pkg,
       goto cleanup_parser;
     }
 
-  if (type == PKG_FILE_TYPE_DATA_SOUND || type == PKG_FILE_TYPE_DATA_PRESET)
+  if (type == PKG_FILE_TYPE_DATA_SOUND)
     {
       pkg->manifest->tags = package_get_tags_from_snd_metadata_int (reader);
     }
@@ -625,7 +638,8 @@ package_receive_pkg_resources (struct package *pkg,
 		   hash, size, sample_path);
       debug_print (1, "Getting sample %s...", sample_path);
 
-      if (download_sample (backend, sample_path, &sample_file, control))
+      if (elektron_download_sample_part (backend, sample_path, &sample_file,
+					 control))
 	{
 	  g_free (sample_path);
 	  error_print ("Error while downloading sample. Continuing...");
@@ -665,14 +679,13 @@ cleanup_reader:
   g_object_unref (reader);
 cleanup_parser:
   g_object_unref (parser);
-get_payload:
   idata_free (&metadata_file);
+get_payload:
   debug_print (1, "Getting payload from %s...", payload_path);
   ret = download_data (backend, payload_path, &payload_file, control);
   if (ret)
     {
       error_print ("Error while downloading payload");
-      ret = -1;
     }
   else
     {
@@ -686,6 +699,7 @@ get_payload:
 	  ret = -1;
 	}
     }
+
   return ret;
 }
 
@@ -701,7 +715,7 @@ package_send_pkg_resources (struct package *pkg, const gchar *payload_path,
   gint64 product_type;
   JsonParser *parser;
   JsonReader *reader;
-  GError *error;
+  GError *error = NULL;
   zip_stat_t zstat;
   zip_error_t zerror;
   zip_file_t *zip_file;
@@ -757,7 +771,7 @@ package_send_pkg_resources (struct package *pkg, const gchar *payload_path,
 
   pkg->resources = g_list_append (pkg->resources, pkg_resource);
 
-  control->parts = 1 + packaget_get_pkg_sample_slots (backend);	// main and sample slots
+  control->parts = 1 + package_get_max_sample_slots (backend);	// main and sample slots
   control->part = 0;
   idata_init (&file, pkg_resource->data, NULL, NULL);
   ret = upload_data (backend, payload_path, &file, control);
