@@ -402,9 +402,8 @@ sds_download_try (struct backend *backend, const gchar *path,
   g_mutex_lock (&control->mutex);
   active = control->active;
   g_mutex_unlock (&control->mutex);
-  control->parts = 1;
-  control->part = 0;
-  job_control_set_progress (control, 0.0);
+
+  job_control_reset (control, 1);
 
   debug_print (1, "Receiving dump data...");
 
@@ -758,9 +757,7 @@ sds_upload (struct backend *backend, const gchar *path, struct idata *sample,
   struct sample_info *sample_info = sample->info;
   GByteArray *input = sample->content;
 
-  control->parts = 1;
-  control->part = 0;
-  job_control_set_progress (control, 0.0);
+  job_control_reset (control, 1);
 
   if (common_slot_get_id_from_path (path, &id))
     {
@@ -969,7 +966,7 @@ sds_next_sample_dentry (struct item_iterator *iter)
 
 static gint
 sds_read_dir (struct backend *backend, struct item_iterator *iter,
-	      const gchar *dir, GSList *extensions)
+	      const gchar *dir, const gchar **extensions)
 {
   struct sds_iterator_data *data;
 
@@ -1056,7 +1053,6 @@ static const struct fs_operations FS_SAMPLES_SDS_8B_OPERATIONS = {
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
   .name = "8b1c",
   .gui_name = "8 bits mono",
-  .ext = "wav",
   .gui_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
@@ -1077,7 +1073,6 @@ static const struct fs_operations FS_SAMPLES_SDS_12B_OPERATIONS = {
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
   .name = "12b1c",
   .gui_name = "12 bits mono",
-  .ext = "wav",
   .gui_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
@@ -1098,7 +1093,6 @@ static const struct fs_operations FS_SAMPLES_SDS_14B_OPERATIONS = {
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
   .name = "14b1c",
   .gui_name = "14 bits mono",
-  .ext = "wav",
   .gui_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
@@ -1119,7 +1113,6 @@ static const struct fs_operations FS_SAMPLES_SDS_16B_OPERATIONS = {
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
   .name = "16b1c",
   .gui_name = "16 bits mono",
-  .ext = "wav",
   .gui_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
@@ -1140,7 +1133,6 @@ static const struct fs_operations FS_SAMPLES_SDS_16B_441_OPERATIONS = {
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
   .name = "44.1k16b1c",
   .gui_name = "44.1 KHz 16 bits mono",
-  .ext = "wav",
   .gui_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
@@ -1161,7 +1153,6 @@ static const struct fs_operations FS_SAMPLES_SDS_16B_32_OPERATIONS = {
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
   .name = "32k16b1c",
   .gui_name = "32 KHz 16 bits mono",
-  .ext = "wav",
   .gui_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
@@ -1182,7 +1173,6 @@ static const struct fs_operations FS_SAMPLES_SDS_16B_16_OPERATIONS = {
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
   .name = "16k16b1c",
   .gui_name = "16 KHz 16 bits mono",
-  .ext = "wav",
   .gui_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
@@ -1203,7 +1193,6 @@ static const struct fs_operations FS_SAMPLES_SDS_16B_8_OPERATIONS = {
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
   .name = "8k16b1c",
   .gui_name = "8 KHz 16 bits mono",
-  .ext = "wav",
   .gui_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
