@@ -26,10 +26,18 @@
 #include "guirecorder.h"
 #include "preferences.h"
 
+enum editor_operation
+{
+  EDITOR_OP_NONE,
+  EDITOR_OP_MOVE_LOOP_START,
+  EDITOR_OP_MOVE_LOOP_END,
+  EDITOR_OP_MOVE_SEL_START,
+  EDITOR_OP_MOVE_SEL_END
+};
+
 struct editor
 {
   struct audio audio;
-  struct preferences *preferences;
   GThread *thread;
   GtkWidget *box;
   GtkWidget *waveform_scrolled_window;
@@ -49,11 +57,12 @@ struct editor
   GtkMenu *menu;
   GtkWidget *play_menuitem;
   GtkWidget *delete_menuitem;
+  GtkWidget *undo_menuitem;
   GtkWidget *save_menuitem;
   GtkDialog *record_dialog;
   struct guirecorder guirecorder;
-  guint zoom;
-  guint operation;
+  gdouble zoom;
+  enum editor_operation operation;
   gboolean dirty;
   gboolean ready;
   struct browser *browser;
@@ -63,7 +72,7 @@ void editor_reset (struct editor *editor, struct browser *browser);
 
 void editor_play_clicked (GtkWidget * object, gpointer data);
 
-void editor_start_load_thread (struct editor *editor);
+void editor_start_load_thread (struct editor *editor, gchar * sample_path);
 
 void editor_stop_load_thread (struct editor *editor);
 
@@ -73,4 +82,6 @@ void editor_destroy (struct editor *);
 
 void editor_set_audio_mono_mix (struct editor *editor);
 
+void editor_reset_audio (struct editor *editor);
+
 #endif
