@@ -27,6 +27,8 @@
 #include <stdint.h>
 #include <inttypes.h>
 #include <stddef.h>
+#include <glib.h>
+#include <glib/gstdio.h>
 #include "backend.h"
 #include "regconn.h"
 #include "regpref.h"
@@ -36,13 +38,43 @@
 
 #define GET_FS_OPS_OFFSET(member) offsetof(struct fs_operations, member)
 #define GET_FS_OPS_FUNC(type,fs,offset) (*(((type *) (((gchar *) fs) + offset))))
-#define CHECK_FS_OPS_FUNC(f) if (!(f)) {return -ENOSYS;}
+#define RETURN_IF_NULL(f) if (!(f)) {return -ENOSYS;}
 
 static struct backend backend;
 static struct job_control control;
 static struct sysex_transfer sysex_transfer;
 static gchar *connector, *fs, *op;
+
 const struct fs_operations *fs_ops;
+const gchar *current_path_progress;
+gboolean same_line_progress;
+
+static void
+complete_progress ()
+{
+  if (same_line_progress)
+    {
+      fprintf (stderr, "\n");
+    }
+}
+
+static void
+print_progress (struct job_control *control)
+{
+  gint progress = control->progress * 100;
+  const gchar *end = same_line_progress ? "\r" : "\n";
+  fprintf (stderr, "%s: %3d %%%s", current_path_progress, progress, end);
+  if (same_line_progress)
+    {
+      fflush (stderr);
+    }
+}
+
+static void
+set_progress_type ()
+{
+  same_line_progress = !debug_level && isatty (fileno (stderr));
+}
 
 static const gchar *
 cli_get_path (const gchar *device_path)
@@ -64,11 +96,10 @@ cli_get_path (const gchar *device_path)
 static gint
 cli_ld ()
 {
-  gint i;
   struct backend_device device;
   GArray *devices = backend_get_devices ();
 
-  for (i = 0; i < devices->len; i++)
+  for (gint i = 0; i < devices->len; i++)
     {
       device = g_array_index (devices, struct backend_device, i);
       printf ("%d: id: %s; name: %s\n", i, device.id, device.name);
@@ -82,19 +113,36 @@ cli_ld ()
 static gint
 cli_connect (const gchar *device_path)
 {
-  gint err, id = (gint) atoi (device_path);
   struct backend_device device;
   GArray *devices = backend_get_devices ();
+  gint err, id;
+  gchar *rem;
 
-  if (!devices->len || id >= devices->len)
+  if (!devices->len)
     {
-      error_print ("Invalid device %d", id);
-      return -ENODEV;
+      error_print ("No devices found");
+      err = -ENODEV;
+      goto end;
+    }
+
+  errno = 0;
+  id = (gint) g_ascii_strtoll (device_path, &rem, 10);
+  if (errno || device_path == rem)
+    {
+      error_print ("Device not provided properly in '%s'", device_path);
+      err = -ENODEV;
+      goto end;
+    }
+
+  if (id >= devices->len)
+    {
+      error_print ("Invalid device '%d'", id);
+      err = -ENODEV;
+      goto end;
     }
 
   device = g_array_index (devices, struct backend_device, id);
   err = backend_init_connector (&backend, &device, connector, NULL);
-  g_array_free (devices, TRUE);
 
   if (!err && fs)
     {
@@ -102,10 +150,12 @@ cli_connect (const gchar *device_path)
       if (!fs_ops)
 	{
 	  error_print ("Invalid filesystem '%s'", fs);
-	  return -EINVAL;
+	  err = -EINVAL;
 	}
     }
 
+end:
+  g_array_free (devices, TRUE);
   return err;
 }
 
@@ -135,13 +185,13 @@ cli_list (int argc, gchar *argv[], int *optind)
       return err;
     }
 
-  path = cli_get_path (device_path);
-
-  CHECK_FS_OPS_FUNC (fs_ops->readdir);
+  RETURN_IF_NULL (fs_ops->readdir);
+  RETURN_IF_NULL (fs_ops->print_item);
 
   active = TRUE;
   sysex_transfer.active = TRUE;
 
+  path = cli_get_path (device_path);
   err = fs_ops->readdir (&backend, &iter, path, NULL);
   if (err)
     {
@@ -162,7 +212,7 @@ cli_list (int argc, gchar *argv[], int *optind)
   return EXIT_SUCCESS;
 }
 
-static int
+static gint
 cli_command_path (int argc, gchar *argv[], int *optind, ssize_t member_offset)
 {
   const gchar *path;
@@ -187,12 +237,11 @@ cli_command_path (int argc, gchar *argv[], int *optind, ssize_t member_offset)
       return err;
     }
 
-  path = cli_get_path (device_path);
-
   f = GET_FS_OPS_FUNC (fs_path_func, fs_ops, member_offset);
-  CHECK_FS_OPS_FUNC (f);
-  err = f (&backend, path);
-  return err;
+  RETURN_IF_NULL (f);
+
+  path = cli_get_path (device_path);
+  return f (&backend, path);
 }
 
 static gint
@@ -241,11 +290,11 @@ cli_command_src_dst (int argc, gchar *argv[], int *optind,
     }
 
   f = GET_FS_OPS_FUNC (fs_src_dst_func, fs_ops, member_offset);
-  CHECK_FS_OPS_FUNC (f);
+  RETURN_IF_NULL (f);
+
   src_path = cli_get_path (device_src_path);
   dst_path = cli_get_path (device_dst_path);
-  err = f (&backend, src_path, dst_path);
-  return err;
+  return f (&backend, src_path, dst_path);
 }
 
 static gint
@@ -287,8 +336,8 @@ cli_command_mv_rename (int argc, gchar *argv[], int *optind)
   src_card = atoi (device_src_path);
   src_path = cli_get_path (device_src_path);
 
-  // If move is implemented, rename must behave the same way.
   f = fs_ops->move;
+  // If move is implemented, rename must behave the same way.
   if (f)
     {
       dst_card = atoi (device_dst_path);
@@ -302,15 +351,15 @@ cli_command_mv_rename (int argc, gchar *argv[], int *optind)
   else
     {
       f = fs_ops->rename;
+      RETURN_IF_NULL (f);
+
       dst_path = device_dst_path;
     }
 
-  CHECK_FS_OPS_FUNC (f);
-  err = f (&backend, src_path, dst_path);
-  return err;
+  return f (&backend, src_path, dst_path);
 }
 
-static int
+static gint
 cli_info (int argc, gchar *argv[], int *optind)
 {
   const gchar *device_path;
@@ -347,7 +396,7 @@ cli_info (int argc, gchar *argv[], int *optind)
   while (e)
     {
       const struct fs_operations *fs_ops = e->data;
-      const gchar *name = fs_ops->name;
+      const gchar *name = GET_CLI_NAME (fs_ops);
       gboolean cli_only = fs_ops->gui_name == NULL;
       printf ("%s%s%s", first ? "" : ", ", name,
 	      cli_only ? " (CLI only)" : "");
@@ -359,7 +408,7 @@ cli_info (int argc, gchar *argv[], int *optind)
   return EXIT_SUCCESS;
 }
 
-static int
+static gint
 cli_df (int argc, gchar *argv[], int *optind)
 {
   const gchar *device_path;
@@ -402,7 +451,7 @@ cli_df (int argc, gchar *argv[], int *optind)
 	  "Used", "Available", "Use%");
 
   err = 0;
-  for (guint8 i = 0; i < G_MAXUINT8; i++)
+  for (guint i = 1; i < G_MAXUINT8; i <<= 1)
     {
       gint v = backend.get_storage_stats (&backend, i, &statfs, path);
       if (v >= 0)
@@ -427,7 +476,7 @@ cli_df (int argc, gchar *argv[], int *optind)
   return err;
 }
 
-static int
+static gint
 cli_upgrade_os (int argc, gchar *argv[], int *optind)
 {
   gint err;
@@ -468,6 +517,8 @@ cli_upgrade_os (int argc, gchar *argv[], int *optind)
       return EXIT_FAILURE;
     }
 
+  RETURN_IF_NULL (backend.upgrade_os);
+
   err = file_load (src_path, &idata, NULL);
   if (err)
     {
@@ -478,7 +529,7 @@ cli_upgrade_os (int argc, gchar *argv[], int *optind)
       sysex_transfer.raw = idata.content;
       sysex_transfer.active = TRUE;
       sysex_transfer.timeout = BE_SYSEX_TIMEOUT_MS;
-      CHECK_FS_OPS_FUNC (backend.upgrade_os);
+
       err = backend.upgrade_os (&backend, &sysex_transfer);
       idata_free (&idata);
     }
@@ -486,13 +537,114 @@ cli_upgrade_os (int argc, gchar *argv[], int *optind)
   return err;
 }
 
-static int
-cli_download (int argc, gchar *argv[], int *optind)
+static gint
+cli_download_item (const gchar *src_path, const gchar *dst_path)
+{
+  gint err;
+  gchar *download_path;
+  struct idata idata;
+
+  RETURN_IF_NULL (fs_ops->download);
+  RETURN_IF_NULL (fs_ops->get_download_path);
+  RETURN_IF_NULL (fs_ops->save);
+
+  control.active = TRUE;
+  control.callback = print_progress;
+  current_path_progress = src_path;
+
+  err = fs_ops->download (&backend, src_path, &idata, &control);
+  if (err)
+    {
+      return err;
+    }
+
+  download_path = fs_ops->get_download_path (&backend, fs_ops, dst_path,
+					     src_path, &idata);
+  if (!download_path)
+    {
+      err = -EINVAL;
+      goto cleanup;
+    }
+
+  err = fs_ops->save (download_path, &idata, &control);
+  g_free (download_path);
+
+cleanup:
+  idata_free (&idata);
+
+  complete_progress ();
+
+  return err;
+}
+
+static gint
+cli_download_dir (const gchar *src_path, const gchar *dst_path)
+{
+  gint err;
+  gboolean active;
+  struct item_iterator iter;
+
+  RETURN_IF_NULL (fs_ops->readdir);
+
+  active = TRUE;
+  sysex_transfer.active = TRUE;
+
+  err = fs_ops->readdir (&backend, &iter, src_path, NULL);
+  if (err)
+    {
+      return err;
+    }
+
+  while (!item_iterator_next (&iter) && active && !err)
+    {
+      gchar *rsrc_path;
+      gchar *filename = item_get_filename (&iter.item, fs_ops->options);
+      rsrc_path = path_chain (PATH_INTERNAL, src_path, filename);
+      g_free (filename);
+
+      if (iter.item.type == ITEM_TYPE_FILE && iter.item.size != 0)	//File and non empty slot
+	{
+	  err = cli_download_item (rsrc_path, dst_path);
+	}
+      else if (iter.item.type == ITEM_TYPE_DIR)
+	{
+	  gchar *rdst_path = path_chain (PATH_SYSTEM, dst_path,
+					 iter.item.name);
+
+	  err = g_mkdir (rdst_path, 0755);
+	  if (err)
+	    {
+	      error_print
+		("Error while creating directory '%s'. Continuing...",
+		 rdst_path);
+	    }
+	  else
+	    {
+	      err = cli_download_dir (rsrc_path, rdst_path);
+	    }
+
+	  g_free (rdst_path);
+	}
+
+      g_free (rsrc_path);
+
+      g_mutex_lock (&sysex_transfer.mutex);
+      active = sysex_transfer.active;
+      g_mutex_unlock (&sysex_transfer.mutex);
+    }
+
+  item_iterator_free (&iter);
+
+  return active ? err : -ECANCELED;
+}
+
+static gint
+cli_download (int argc, gchar *argv[], int *optind, gint recursive)
 {
   const gchar *src_path;
-  gchar *device_src_path, *download_path;
+  const gchar *dst_path;
+  gchar *device_src_path;
   gint err;
-  struct idata idata;
 
   if (*optind == argc)
     {
@@ -513,37 +665,96 @@ cli_download (int argc, gchar *argv[], int *optind)
 
   src_path = cli_get_path (device_src_path);
 
+  if (*optind == argc)
+    {
+      dst_path = ".";
+    }
+  else
+    {
+      dst_path = argv[*optind];
+
+      debug_print (1, "Creating directory '%s'...", dst_path);
+      err = g_mkdir_with_parents (dst_path, 0755);
+      if (err)
+	{
+	  error_print ("Error while creating directory '%s'", dst_path);
+	  return err;
+	}
+    }
+
+  if (recursive)
+    {
+      if (strcmp (src_path, "/"))
+	{
+	  gchar *new_dir = g_path_get_basename (src_path);
+	  gchar *full_dst_path = path_chain (PATH_SYSTEM, dst_path, new_dir);
+	  debug_print (1, "Creating directory '%s'...", full_dst_path);
+	  err = g_mkdir (full_dst_path, 0755);
+	  if (err)
+	    {
+	      error_print ("Error while creating directory '%s'",
+			   full_dst_path);
+	    }
+	  else
+	    {
+	      err = cli_download_dir (src_path, full_dst_path);
+	    }
+
+	  g_free (full_dst_path);
+	  g_free (new_dir);
+	  return err;
+	}
+      else
+	{
+	  return cli_download_dir (src_path, dst_path);
+	}
+    }
+  else
+    {
+      return cli_download_item (src_path, dst_path);
+    }
+}
+
+static gint
+cli_upload_item (const gchar *src_path, const gchar *dst_path)
+{
+  gint err;
+  gchar *upload_path;
+  struct idata idata;
+
+  RETURN_IF_NULL (fs_ops->load);
+  RETURN_IF_NULL (fs_ops->get_upload_path);
+  RETURN_IF_NULL (fs_ops->upload);
+
   control.active = TRUE;
-  CHECK_FS_OPS_FUNC (fs_ops->download);
-  err = fs_ops->download (&backend, src_path, &idata, &control);
+  control.callback = print_progress;
+  current_path_progress = src_path;
+
+  err = fs_ops->load (src_path, &idata, &control);
   if (err)
     {
       return err;
     }
 
-  download_path = fs_ops->get_download_path (&backend, fs_ops, ".", src_path,
-					     &idata);
-  if (!download_path)
-    {
-      err = -EINVAL;
-      goto cleanup;
-    }
+  upload_path = fs_ops->get_upload_path (&backend, fs_ops, dst_path,
+					 src_path, &idata);
 
-  err = fs_ops->save (download_path, &idata, &control);
-  g_free (download_path);
-
-cleanup:
+  err = fs_ops->upload (&backend, upload_path, &idata, &control);
   idata_free (&idata);
+
+  g_free (upload_path);
+
+  complete_progress ();
+
   return err;
 }
 
-static int
+static gint
 cli_upload (int argc, gchar *argv[], int *optind)
 {
-  const gchar *dst_path;
-  gchar *src_path, *device_dst_path, *upload_path;
   gint err;
-  struct idata idata;
+  const gchar *dst_path;
+  gchar *src_path, *device_dst_path;
 
   if (*optind == argc)
     {
@@ -575,26 +786,10 @@ cli_upload (int argc, gchar *argv[], int *optind)
 
   dst_path = cli_get_path (device_dst_path);
 
-  control.active = TRUE;
-  err = fs_ops->load (src_path, &idata, &control);
-  if (err)
-    {
-      return err;
-    }
-
-  upload_path = fs_ops->get_upload_path (&backend, fs_ops, dst_path,
-					 src_path, &idata);
-
-  CHECK_FS_OPS_FUNC (fs_ops->upload);
-  err = fs_ops->upload (&backend, upload_path, &idata, &control);
-  idata_free (&idata);
-
-  g_free (upload_path);
-
-  return err;
+  return cli_upload_item (src_path, dst_path);
 }
 
-static int
+static gint
 cli_send (int argc, gchar *argv[], int *optind)
 {
   gint err;
@@ -648,7 +843,7 @@ cli_send (int argc, gchar *argv[], int *optind)
   return err;
 }
 
-static int
+static gint
 cli_receive (int argc, gchar *argv[], int *optind)
 {
   gint err;
@@ -756,6 +951,7 @@ main (int argc, gchar *argv[])
   gint err;
   gchar *command;
   gint vflg = 0, errflg = 0;
+
 #if defined(__linux__)
   struct sigaction action;
 
@@ -803,6 +999,8 @@ main (int argc, gchar *argv[])
       exit (EXIT_FAILURE);
     }
 
+  set_progress_type ();
+
   regconn_register ();
   regpref_register ();
   preferences_load ();
@@ -859,7 +1057,12 @@ main (int argc, gchar *argv[])
 	}
       else if (!strcmp (op, "download") || !strcmp (op, "dl"))
 	{
-	  err = cli_download (argc, argv, &optind);
+	  err = cli_download (argc, argv, &optind, 0);
+	}
+      else if (!strcmp (op, "rdownload") || !strcmp (op, "rdl") ||
+	       !strcmp (op, "backup"))
+	{
+	  err = cli_download (argc, argv, &optind, 1);
 	}
       else if (!strcmp (op, "upload") || !strcmp (op, "ul"))
 	{
