@@ -85,13 +85,6 @@ static gpointer elektroid_upload_task_runner (gpointer);
 static gpointer elektroid_download_task_runner (gpointer);
 static void elektroid_update_progress (struct job_control *);
 
-static const struct option ELEKTROID_OPTIONS[] = {
-  {"local-directory", 1, NULL, 'l'},
-  {"verbose", 0, NULL, 'v'},
-  {"help", 0, NULL, 'h'},
-  {NULL, 0, NULL, 0}
-};
-
 static const GtkTargetEntry TARGET_ENTRIES_LOCAL_DST[] = {
   {TEXT_URI_LIST_STD, 0, TARGET_STRING},
   {TEXT_URI_LIST_ELEKTROID, GTK_TARGET_SAME_APP | GTK_TARGET_OTHER_WIDGET,
@@ -130,6 +123,7 @@ static const GtkTargetEntry TARGET_ENTRIES_UP_BUTTON_DST[] = {
 };
 
 static const gchar *hostname;
+static gchar *local_dir;
 
 struct editor editor;
 struct tasks tasks;
@@ -141,9 +135,10 @@ static struct backend backend;
 
 static guint batch_id;
 
-GtkWindow *main_window;
 GtkWidget *dialog;
 
+static GtkWindow *main_window;
+static GtkBuilder *builder;
 static GtkAboutDialog *about_dialog;
 static GtkDialog *name_dialog;
 static GtkEntry *name_dialog_entry;
@@ -273,7 +268,7 @@ elektroid_update_backend_status ()
       statfss = g_string_new (NULL);
       if (backend.get_storage_stats)
 	{
-	  for (guint8 i = 0; i < G_MAXUINT8; i++)
+	  for (guint i = 1; i < G_MAXUINT8; i <<= 1)
 	    {
 	      gint v = backend.get_storage_stats (&backend, i, &statfs,
 						  remote_browser.dir);
@@ -442,8 +437,9 @@ elektroid_rx_sysex ()
   gint *res;
   GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_SAVE;
 
-  res = progress_run (elektroid_rx_sysex_runner, PROGRESS_TYPE_UPDATE, NULL,
-		      _("Receiving SysEx"), "", TRUE, &dres);
+  res =
+    progress_run (elektroid_rx_sysex_runner, PROGRESS_TYPE_SYSEX_TRANSFER,
+		  NULL, _("Receiving SysEx"), "", TRUE, &dres);
   if (!res)			//Signal captured while running the dialog.
     {
       g_byte_array_free (progress.sysex_transfer.raw, TRUE);
@@ -612,7 +608,7 @@ elektroid_tx_sysex_common (GThreadFunc func, gboolean multiple)
     {
       gtk_widget_hide (GTK_WIDGET (dialog));
       filenames = gtk_file_chooser_get_filenames (chooser);
-      err = progress_run (func, PROGRESS_TYPE_UPDATE, filenames,
+      err = progress_run (func, PROGRESS_TYPE_SYSEX_TRANSFER, filenames,
 			  _("Sending SysEx"), "", TRUE, NULL);
       g_slist_free_full (g_steal_pointer (&filenames), g_free);
 
@@ -747,7 +743,7 @@ elektroid_delete_file (struct browser *browser, gchar *dir, struct item *item)
 
   if (item->type == ITEM_TYPE_FILE)
     {
-      gchar *filename = get_filename (browser->fs_ops->options, item);
+      gchar *filename = item_get_filename (item, browser->fs_ops->options);
       gchar *id_path = path_chain (type, dir, filename);
       g_free (filename);
       err = browser->fs_ops->delete (browser->backend, id_path);
@@ -887,7 +883,7 @@ elektroid_rename_item (GtkWidget *object, gpointer data)
 {
   gchar *old_path, *new_path;
   const gchar *ext;
-  gint result, err, sel_len;
+  gint result, err, sel_len, ext_len;
   GtkTreeIter iter;
   struct item item;
   struct browser *browser = data;
@@ -900,7 +896,11 @@ elektroid_rename_item (GtkWidget *object, gpointer data)
 
   sel_len = strlen (item.name);
   ext = filename_get_ext (item.name);
-  sel_len -= strlen (ext) + 1;
+  ext_len = strlen (ext);
+  if (ext_len)
+    {
+      sel_len -= ext_len + 1;
+    }
 
   gtk_entry_set_max_length (name_dialog_entry, browser->fs_ops->max_name_len);
   gtk_entry_set_text (name_dialog_entry, item.name);
@@ -1242,8 +1242,9 @@ elektroid_run_dialog_and_destroy (GtkWidget *custom_dialog)
 }
 
 gchar *
-elektroid_ask_name (const gchar *title, const gchar *value,
-		    struct browser *browser, gint start_pos, gint end_pos)
+elektroid_ask_name_get_path (const gchar *title, const gchar *value,
+			     struct browser *browser, gint start_pos,
+			     gint end_pos)
 {
   char *pathname = NULL;
   int result;
@@ -1285,7 +1286,8 @@ elektroid_add_dir (GtkWidget *object, gpointer data)
   char *pathname;
   struct browser *browser = data;
 
-  pathname = elektroid_ask_name (_("Add Directory"), "", browser, 0, 0);
+  pathname = elektroid_ask_name_get_path (_("Add Directory"), "", browser, 0,
+					  0);
   if (pathname)
     {
       gint err = browser->fs_ops->mkdir (&backend, pathname);
@@ -1788,7 +1790,8 @@ elektroid_add_download_task_path (const gchar *rel_path,
 
   while (!item_iterator_next (&iter))
     {
-      filename = get_filename (remote_browser.fs_ops->options, &iter.item);
+      filename = item_get_filename (&iter.item,
+				    remote_browser.fs_ops->options);
       path = path_chain (PATH_INTERNAL, rel_path, filename);
       elektroid_add_download_task_path (path, src_dir, dst_dir);
       debug_print (1, "name: %s", filename);
@@ -1826,7 +1829,7 @@ elektroid_add_download_tasks_runner (gpointer data)
 
       gtk_tree_model_get_iter (model, &path_iter, path);
       browser_set_item (model, &path_iter, &item);
-      filename = get_filename (remote_browser.fs_ops->options, &item);
+      filename = item_get_filename (&item, remote_browser.fs_ops->options);
       elektroid_add_download_task_path (filename, remote_browser.dir,
 					local_browser.dir);
       g_free (filename);
@@ -2310,7 +2313,7 @@ elektroid_add_upload_task_slot (const gchar *name,
 
       browser_set_item (model, &iter, &item);
 
-      filename = get_filename (remote_browser.fs_ops->options, &item);
+      filename = item_get_filename (&item, remote_browser.fs_ops->options);
       str = g_string_new (NULL);
       g_string_append_printf (str, "%s%s%s", remote_browser.dir,
 			      strcmp (remote_browser.dir, "/") ?
@@ -2423,6 +2426,15 @@ elektroid_dnd_received (GtkWidget *widget, GdkDragContext *context,
       return;
     }
 
+  if (widget == GTK_WIDGET (remote_browser.view) &&
+      (remote_browser.fs_ops->options & FS_OPTION_SLOT_STORAGE) &&
+      remote_browser.dnd_motion_path == NULL)
+    {
+      gtk_drag_finish (context, TRUE, TRUE, time);
+      error_print ("DND destination needs a slot");
+      return;
+    }
+
   dnd_data = g_malloc (sizeof (struct elektroid_dnd_data));
   dnd_data->widget = widget;
 
@@ -2579,7 +2591,6 @@ elektroid_drag_motion_list (GtkWidget *widget,
   gint tx, ty;
   gboolean slot;
   GtkTreeSelection *selection;
-  GtkTreeViewColumn *column;
   struct item item;
   struct browser *browser = user_data;
 
@@ -2589,46 +2600,44 @@ elektroid_drag_motion_list (GtkWidget *widget,
   gtk_tree_view_convert_widget_to_bin_window_coords (browser->view, wx, wy,
 						     &tx, &ty);
 
-  if (gtk_tree_view_get_path_at_pos (browser->view, tx, ty, &path, &column,
+  if (gtk_tree_view_get_path_at_pos (browser->view, tx, ty, &path, NULL,
 				     NULL, NULL))
     {
       GtkAllocation allocation;
       gtk_widget_get_allocation (widget, &allocation);
-      if (column == browser->tree_view_name_column)
+
+      spath = gtk_tree_path_to_string (path);
+      debug_print (2, "Drag motion path: %s", spath);
+      g_free (spath);
+
+      if (slot)
 	{
-	  spath = gtk_tree_path_to_string (path);
-	  debug_print (2, "Drag motion path: %s", spath);
-	  g_free (spath);
-
-	  if (slot)
+	  gtk_tree_view_set_drag_dest_row (browser->view, path,
+					   GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
+	}
+      else
+	{
+	  selection = gtk_tree_view_get_selection (browser->view);
+	  if (gtk_tree_selection_path_is_selected (selection, path))
 	    {
-	      gtk_tree_view_set_drag_dest_row (remote_browser.view, path,
-					       GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
-	    }
-	  else
-	    {
-	      selection = gtk_tree_view_get_selection (browser->view);
-	      if (gtk_tree_selection_path_is_selected (selection, path))
-		{
-		  browser_clear_dnd_function (browser);
-		  return TRUE;
-		}
-	    }
-
-	  model = gtk_tree_view_get_model (browser->view);
-	  gtk_tree_model_get_iter (model, &iter, path);
-	  browser_set_item (model, &iter, &item);
-
-	  if (item.type == ITEM_TYPE_DIR && (!browser->dnd_motion_path ||
-					     (browser->dnd_motion_path &&
-					      gtk_tree_path_compare
-					      (browser->dnd_motion_path,
-					       path))))
-	    {
-	      browser_set_dnd_function (browser, elektroid_drag_list_timeout);
+	      browser_clear_dnd_function (browser);
+	      return TRUE;
 	    }
 	}
-      else if (ty < TREEVIEW_EDGE_SIZE)
+
+      model = gtk_tree_view_get_model (browser->view);
+      gtk_tree_model_get_iter (model, &iter, path);
+      browser_set_item (model, &iter, &item);
+
+      if (item.type == ITEM_TYPE_DIR && (!browser->dnd_motion_path ||
+					 (browser->dnd_motion_path &&
+					  gtk_tree_path_compare
+					  (browser->dnd_motion_path, path))))
+	{
+	  browser_set_dnd_function (browser, elektroid_drag_list_timeout);
+	}
+
+      if (ty < TREEVIEW_EDGE_SIZE)
 	{
 	  browser_set_dnd_function (browser,
 				    elektroid_drag_scroll_up_timeout);
@@ -2638,14 +2647,15 @@ elektroid_drag_motion_list (GtkWidget *widget,
 	  browser_set_dnd_function (browser,
 				    elektroid_drag_scroll_down_timeout);
 	}
-      else
-	{
-	  browser_clear_dnd_function (browser);
-	}
     }
   else
     {
       browser_clear_dnd_function (browser);
+      if (slot)
+	{
+	  gtk_tree_view_set_drag_dest_row (browser->view, NULL,
+					   GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
+	}
     }
 
   if (browser->dnd_motion_path)
@@ -2710,7 +2720,7 @@ elektroid_set_window_size ()
 }
 
 static void
-elektroid_quit ()
+elektroid_exit ()
 {
   gtk_dialog_response (GTK_DIALOG (about_dialog), GTK_RESPONSE_CANCEL);
   progress_response (GTK_RESPONSE_CANCEL);
@@ -2728,25 +2738,29 @@ elektroid_quit ()
 
   editor_destroy (&editor);
 
-  debug_print (1, "Quitting GTK+...");
-  gtk_main_quit ();
+  if (backend_check (&backend))
+    {
+      backend_destroy (&backend);
+    }
+
+  gtk_widget_destroy (GTK_WIDGET (main_window));
+  g_object_unref (builder);
 }
 
 static gboolean
 elektroid_delete_window (GtkWidget *widget, GdkEvent *event, gpointer data)
 {
-  elektroid_quit ();
+  elektroid_exit ();
   return FALSE;
 }
 
-static int
-elektroid_run (int argc, char *argv[])
+static void
+build_ui ()
 {
-  GtkBuilder *builder;
   GtkCssProvider *css_provider;
   GtkWidget *refresh_devices_button;
+  gchar *thanks;
 
-  gtk_init (&argc, &argv);
   builder = gtk_builder_new ();
   gtk_builder_add_from_file (builder, DATADIR "/gui.glade", NULL);
 
@@ -2763,6 +2777,21 @@ elektroid_run (int argc, char *argv[])
     GTK_ABOUT_DIALOG (gtk_builder_get_object (builder, "about_dialog"));
   gtk_about_dialog_set_version (about_dialog, PACKAGE_VERSION);
 
+  if (g_file_get_contents (DATADIR "/THANKS", &thanks, NULL, NULL))
+    {
+      gchar *last_new_line = strrchr (thanks, '\n');
+      if (last_new_line != NULL)
+	{
+	  *last_new_line = 0;
+	}
+      gchar **lines = g_strsplit (thanks, "\n", 0);
+      gtk_about_dialog_add_credit_section (about_dialog,
+					   _("Acknowledgements"),
+					   (const gchar **) lines);
+      g_free (thanks);
+      g_strfreev (lines);
+    }
+
   name_dialog = GTK_DIALOG (gtk_builder_get_object (builder, "name_dialog"));
   name_dialog_accept_button =
     GTK_WIDGET (gtk_builder_get_object
@@ -3000,111 +3029,86 @@ elektroid_run (int argc, char *argv[])
   gtk_entry_set_text (GTK_ENTRY (local_name_entry), hostname);
 
   elektroid_set_window_size ();
-
-  gtk_widget_show (GTK_WIDGET (main_window));
-
-  gtk_main ();
-
-  preferences_set_string (PREF_KEY_LOCAL_DIR, strdup (local_browser.dir));
-  elektroid_set_preferences_remote_dir ();
-
-  if (backend_check (&backend))
-    {
-      backend_destroy (&backend);
-    }
-
-  g_object_unref (G_OBJECT (builder));
-
-  return EXIT_SUCCESS;
 }
 
 #if defined(__linux__)
 static gboolean
-elektroid_end (gpointer data)
+elektroid_signal_handler (gpointer data)
 {
-  elektroid_quit ();
+  elektroid_exit ();
   return FALSE;
 }
 #endif
 
 static void
-elektroid_print_help (gchar *executable_path)
+elektroid_startup (GApplication *gapp, gpointer *user_data)
 {
-  gchar *exec_name;
-  const struct option *option;
-
-  fprintf (stderr, "%s\n", PACKAGE_STRING);
-  exec_name = g_path_get_basename (executable_path);
-  fprintf (stderr, "Usage: %s [options]\n", exec_name);
-  fprintf (stderr, "Options:\n");
-  option = ELEKTROID_OPTIONS;
-  while (option->name)
+  if (local_dir)
     {
-      fprintf (stderr, "  --%s, -%c", option->name, option->val);
-      if (option->has_arg)
-	{
-	  fprintf (stderr, " value");
-	}
-      fprintf (stderr, "\n");
-      option++;
+      preferences_set_string (PREF_KEY_LOCAL_DIR,
+			      get_system_startup_path (local_dir));
+      g_free (local_dir);
     }
-  g_free (exec_name);
+
+  build_ui ();
+  gtk_application_add_window (GTK_APPLICATION (gapp),
+			      GTK_WINDOW (main_window));
 }
 
+static void
+elektroid_activate (GApplication *gapp, gpointer *user_data)
+{
+  gtk_window_present (GTK_WINDOW (main_window));
+}
+
+static gboolean
+elektroid_increment_debug_level (const gchar *option_name,
+				 const gchar *value,
+				 gpointer data, GError **error)
+{
+  debug_level++;
+  return TRUE;
+}
+
+const GOptionEntry CMD_PARAMS[] = {
+  {
+   .long_name = "verbosity",
+   .short_name = 'v',
+   .flags = G_OPTION_FLAG_NO_ARG,
+   .arg = G_OPTION_ARG_CALLBACK,
+   .arg_data = elektroid_increment_debug_level,
+   .description =
+   "Increase verbosity. For more verbosity use it more than once.",
+   .arg_description = NULL,
+   },
+  {
+   .long_name = "local-directory",
+   .short_name = 'l',
+   .flags = G_OPTION_FLAG_NONE,
+   .arg = G_OPTION_ARG_FILENAME,
+   .arg_data = &local_dir,
+   .description = "Local directory at startup",
+   .arg_description = "DIRECTORY",
+   },
+  {NULL}
+};
+
 int
 main (int argc, char *argv[])
 {
-  gint opt, ret;
-  gchar *local_dir = NULL;
-  gint vflg = 0, dflg = 0, errflg = 0;
-  int long_index = 0;
+  gint err;
+  GtkApplication *app;
 
 #if defined(__linux__)
-  g_unix_signal_add (SIGHUP, elektroid_end, NULL);
-  g_unix_signal_add (SIGINT, elektroid_end, NULL);
-  g_unix_signal_add (SIGTERM, elektroid_end, NULL);
+  g_unix_signal_add (SIGHUP, elektroid_signal_handler, NULL);
+  g_unix_signal_add (SIGINT, elektroid_signal_handler, NULL);
+  g_unix_signal_add (SIGTERM, elektroid_signal_handler, NULL);
 #endif
 
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
-  while ((opt = getopt_long (argc, argv, "l:vh", ELEKTROID_OPTIONS,
-			     &long_index)) != -1)
-    {
-      switch (opt)
-	{
-	case 'l':
-	  local_dir = optarg;
-	  dflg++;
-	  break;
-	case 'v':
-	  vflg++;
-	  break;
-	case 'h':
-	  elektroid_print_help (argv[0]);
-	  exit (EXIT_SUCCESS);
-	case '?':
-	  errflg++;
-	}
-    }
-
-  if (dflg > 1)
-    {
-      errflg++;
-    }
-
-  if (vflg)
-    {
-      debug_level = vflg;
-    }
-
-  if (errflg > 0)
-    {
-      elektroid_print_help (argv[0]);
-      exit (EXIT_FAILURE);
-    }
-
   hostname = g_get_host_name ();
 
   regconn_register ();
@@ -3112,13 +3116,21 @@ main (int argc, char *argv[])
   regpref_register ();
 
   preferences_load ();
-  if (local_dir)
-    {
-      preferences_set_string (PREF_KEY_LOCAL_DIR,
-			      get_system_startup_path (local_dir));
-    }
 
-  ret = elektroid_run (argc, argv);
+  app = gtk_application_new ("io.github.dagargo.Elektroid",
+			     G_APPLICATION_NON_UNIQUE);
+
+  g_signal_connect (app, "startup", G_CALLBACK (elektroid_startup), NULL);
+  g_signal_connect (app, "activate", G_CALLBACK (elektroid_activate), NULL);
+
+  g_application_add_main_option_entries (G_APPLICATION (app), CMD_PARAMS);
+
+  err = g_application_run (G_APPLICATION (app), argc, argv);
+
+  g_object_unref (app);
+
+  preferences_set_string (PREF_KEY_LOCAL_DIR, strdup (local_browser.dir));
+  elektroid_set_preferences_remote_dir ();
 
   preferences_save ();
   preferences_free ();
@@ -3127,5 +3139,5 @@ main (int argc, char *argv[])
   regma_unregister ();
   regpref_unregister ();
 
-  return ret;
+  return err;
 }
