@@ -22,7 +22,8 @@
 #define GUIRECORDER_H
 
 #include <gtk/gtk.h>
-#include "utils.h"
+#include "connector.h"
+#include "audio.h"
 
 enum channels_list_store_columns
 {
@@ -35,15 +36,16 @@ struct guirecorder
   GtkWidget *channels_combo;
   GtkListStore *channels_list_store;
   GtkLevelBar *monitor_levelbar;
+  struct audio *audio;
   gdouble level;
 };
 
 void guirecorder_monitor_notifier (void *, gdouble);
 
-guint guirecorder_get_channel_mask (GtkWidget *);
+guint guirecorder_get_channel_mask (struct guirecorder *guirecorder);
 
-void guirecorder_set_channels_masks (struct guirecorder *,
-				     const struct fs_operations *);
+void guirecorder_set_channels_masks (struct guirecorder *guirecorder,
+				     guint32);
 
 void guirecorder_channels_changed (GtkWidget *, gpointer);
 
