@@ -22,11 +22,10 @@
 #include "connectors/system.h"
 
 const struct fs_operations FS_LOCAL_GENERIC_OPERATIONS = {
-  .fs = 0,
-  .options = FS_OPTION_SORT_BY_NAME | FS_OPTION_ALLOW_SEARCH,
+  .options = FS_OPTION_ALLOW_SEARCH,
   .name = "local",
   .gui_name = "localhost",
-  .gui_icon = BE_FILE_ICON_GENERIC,
+  .gui_icon = FS_ICON_GENERIC,
   .readdir = system_read_dir,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
@@ -37,21 +36,17 @@ const struct fs_operations FS_LOCAL_GENERIC_OPERATIONS = {
 };
 
 const struct fs_operations FS_LOCAL_SAMPLE_OPERATIONS = {
-  .fs = 0,
-  .options = FS_OPTION_MONO | FS_OPTION_STEREO | FS_OPTION_SORT_BY_NAME |
-    FS_OPTION_SAMPLE_ATTRS | FS_OPTION_ALLOW_SEARCH,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_STEREO |
+    FS_OPTION_SHOW_SAMPLE_COLUMNS | FS_OPTION_ALLOW_SEARCH,
   .name = "local",
   .gui_name = "localhost",
-  .gui_icon = BE_FILE_ICON_WAVE,
+  .gui_icon = FS_ICON_WAVE,
   .readdir = system_samples_read_dir,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
   .delete = system_delete,
   .rename = system_rename,
   .move = system_rename,
-  //While the local operations do not need to implement download, upload, load or save, the upload function is used by the editor to save the loaded sample in the appropriate format.
-  .upload = system_upload,
-  .get_ext = backend_get_fs_ext,
-  .type_ext = "wav",
+  .get_exts = sample_get_sample_extensions,
   .max_name_len = 255
 };
