@@ -21,8 +21,8 @@
 #ifndef LOCAL_H
 #define LOCAL_H
 
-#include "backend.h"
-#include "utils.h"
+#include "connector.h"
+#include "sample.h"
 
 extern const struct fs_operations FS_LOCAL_GENERIC_OPERATIONS;
 extern const struct fs_operations FS_LOCAL_SAMPLE_OPERATIONS;
