@@ -21,30 +21,21 @@
 #ifndef NOTIFIER_H
 #define NOTIFIER_H
 
-#if defined(__linux__)
-#include <sys/inotify.h>
-#endif
-#include <gtk/gtk.h>
+#include <glib.h>
 #include "browser.h"
 
 struct notifier
 {
-#if defined(__linux__)
-  gchar *dir;
-  gint fd;
-  gint wd;
-  size_t event_size;
-  struct inotify_event *event;
+  GFile *dir;
+  GFileMonitor *monitor;
+  GFileMonitor *parent_monitor;
   struct browser *browser;
-  GThread *thread;
-  GMutex mutex;
-#endif
 };
 
-void notifier_init (struct notifier *, struct browser *);
+void notifier_init (struct notifier **notifier, struct browser *browser);
 
-void notifier_set_active (struct notifier *, gboolean);
+void notifier_update_dir (struct notifier *, gboolean active);
 
-void notifier_destroy (struct notifier *);
+void notifier_destroy (struct notifier *notifier);
 
 #endif
