@@ -122,7 +122,7 @@ void job_control_set_progress_no_sync (struct job_control *control,
 void job_control_set_progress (struct job_control *control, gdouble progress);
 
 gboolean file_matches_extensions (const gchar * name,
-				  const GSList * extensions);
+				  const gchar ** extensions);
 
 gchar *path_chain (enum path_type, const gchar *, const gchar *);
 
@@ -146,4 +146,6 @@ gboolean job_control_get_active_lock (struct job_control *control);
 void job_control_set_active_lock (struct job_control *control,
 				  gboolean active);
 
+void job_control_reset (struct job_control *control, gint parts);
+
 #endif
