@@ -56,6 +56,26 @@ test_common_get_sanitized_name ()
   g_free (str);
 }
 
+void
+test_common_to_os_sanitized_name ()
+{
+  gchar *str = strdup ("\\^/");
+
+  printf ("\n");
+
+  common_to_os_sanitized_name (str);
+  CU_ASSERT_STRING_EQUAL (str, "?^?");
+  g_free (str);
+}
+
+static const gchar *EXTS[] = { "ext", NULL };
+
+static const gchar **
+get_exts ()
+{
+  return EXTS;
+}
+
 void
 test_common_slot_get_download_path ()
 {
@@ -66,7 +86,7 @@ test_common_slot_get_download_path ()
 
   snprintf (backend.name, LABEL_MAX, "Dev Name");
   ops.name = "fsname";
-  ops.ext = "ext";
+  ops.get_exts = get_exts;
 
   idata_init (&idata, NULL, strdup ("name"), NULL);
 
@@ -132,6 +152,12 @@ main (gint argc, gchar *argv[])
       goto cleanup;
     }
 
+  if (!CU_add_test (suite, "test_common_to_os_sanitized_name",
+		    test_common_to_os_sanitized_name))
+    {
+      goto cleanup;
+    }
+
   if (!CU_add_test (suite, "common_slot_get_download_path",
 		    test_common_slot_get_download_path))
     {
